@@ -23,6 +23,7 @@ import davmail.Settings;
 import davmail.exception.DavMailAuthenticationException;
 import davmail.exception.DavMailException;
 import davmail.exception.WebdavNotAvailableException;
+import davmail.exchange.auth.ExchangeAuthenticator;
 import davmail.exchange.dav.DavExchangeSession;
 import davmail.exchange.ews.EwsExchangeSession;
 import davmail.http.DavGatewayHttpClientFacade;
@@ -30,6 +31,7 @@ import org.apache.commons.httpclient.HttpClient;
 import org.apache.commons.httpclient.HttpStatus;
 import org.apache.commons.httpclient.methods.GetMethod;
 
+import java.awt.*;
 import java.io.IOException;
 import java.net.NetworkInterface;
 import java.net.SocketException;
@@ -139,14 +141,50 @@ public final class ExchangeSessionFactory {
             }
 
             if (session == null) {
-                String enableEws = Settings.getProperty("davmail.enableEws", "auto");
-                if ("true".equals(enableEws) || poolKey.url.toLowerCase().endsWith("/ews/exchange.asmx")) {
-                    session = new EwsExchangeSession(poolKey.url, poolKey.userName, poolKey.password);
+                String mode = Settings.getProperty("davmail.mode");
+                // convert old setting
+                if (mode == null) {
+                    if ("false".equals(Settings.getProperty("davmail.enableEws"))) {
+                        mode = Settings.WEBDAV;
+                    } else {
+                        mode = Settings.EWS;
+                    }
+                }
+                // check for overridden authenticator
+                String authenticatorClass = Settings.getProperty("davmail.authenticator");
+                if (authenticatorClass == null) {
+                    if (Settings.O365_MODERN.equals(mode)) {
+                        authenticatorClass = "davmail.exchange.auth.O365Authenticator";
+                    } else if (Settings.O365_INTERACTIVE.equals(mode)) {
+                        authenticatorClass = "davmail.exchange.auth.O365InteractiveAuthenticator";
+                        if (GraphicsEnvironment.isHeadless()) {
+                            throw new DavMailException("EXCEPTION_DAVMAIL_CONFIGURATION", "O365Interactive not supported in headless mode");
+                        }
+                    }
+                }
+
+                if (authenticatorClass != null) {
+                    ExchangeAuthenticator authenticator = (ExchangeAuthenticator) Class.forName(authenticatorClass).newInstance();
+                    authenticator.setUsername(userName);
+                    authenticator.setPassword(password);
+                    authenticator.authenticate();
+                    HttpClient httpClient = DavGatewayHttpClientFacade.getInstance(authenticator.getEWSUrl());
+                    session = new EwsExchangeSession(httpClient, userName);
+                    ((EwsExchangeSession) session).setToken(authenticator.getToken());
+                    // TODO: refactor buildSessionInfo
+                    session.buildSessionInfo(null);
+
+                } else if (Settings.EWS.equals(mode) || poolKey.url.toLowerCase().endsWith("/ews/exchange.asmx")) {
+                    HttpClient httpClient = DavGatewayHttpClientFacade.getInstance(poolKey.url);
+                    DavGatewayHttpClientFacade.setCredentials(httpClient, poolKey.userName, poolKey.password);
+                    DavGatewayHttpClientFacade.addNTLM(httpClient);
+                    session = new EwsExchangeSession(httpClient, poolKey.userName);
+                    session.buildSessionInfo(null);
                 } else {
                     try {
                         session = new DavExchangeSession(poolKey.url, poolKey.userName, poolKey.password);
                     } catch (WebdavNotAvailableException e) {
-                        if ("auto".equals(enableEws)) {
+                        if (Settings.AUTO.equals(mode)) {
                             ExchangeSession.LOGGER.debug(e.getMessage() + ", retry with EWS");
                             session = new EwsExchangeSession(poolKey.url, poolKey.userName, poolKey.password);
                         } else {
