@@ -46,6 +46,7 @@ import org.apache.log4j.Logger;
 
 import java.io.IOException;
 import java.net.*;
+import java.security.Security;
 import java.util.*;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
@@ -56,7 +57,7 @@ import java.util.regex.Pattern;
 public final class DavGatewayHttpClientFacade {
     static final Logger LOGGER = Logger.getLogger("davmail.http.DavGatewayHttpClientFacade");
 
-    static final String IE_USER_AGENT = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0)";
+    public static final String IE_USER_AGENT = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0)";
     static final int MAX_REDIRECTS = 10;
     static final Object LOCK = new Object();
     private static boolean needNTLM;
@@ -75,6 +76,8 @@ public final class DavGatewayHttpClientFacade {
         // force strong ephemeral Diffie-Hellman parameter
         System.setProperty("jdk.tls.ephemeralDHKeySize", "2048");
 
+        Security.setProperty("ssl.SocketFactory.provider", "davmail.http.DavGatewaySSLSocketFactory");
+
         DavGatewayHttpClientFacade.start();
 
         // register custom cookie policy
@@ -86,6 +89,12 @@ public final class DavGatewayHttpClientFacade {
         } catch (Throwable t) {
             // ignore
         }
+
+        // set system property *before* calling ProxySelector.getDefault()
+        if (Settings.getBooleanProperty("davmail.useSystemProxies", Boolean.FALSE)) {
+            System.setProperty("java.net.useSystemProxies", "true");
+        }
+        ProxySelector.setDefault(new DavGatewayProxySelector(ProxySelector.getDefault()));
     }
 
 
@@ -99,7 +108,7 @@ public final class DavGatewayHttpClientFacade {
      */
     private static HttpClient getBaseInstance() {
         HttpClient httpClient = new HttpClient();
-        httpClient.getParams().setParameter(HttpMethodParams.USER_AGENT, IE_USER_AGENT);
+        httpClient.getParams().setParameter(HttpMethodParams.USER_AGENT, getUserAgent());
         httpClient.getParams().setParameter(HttpClientParams.MAX_REDIRECTS, Settings.getIntProperty("davmail.httpMaxRedirects", MAX_REDIRECTS));
         httpClient.getParams().setCookiePolicy("DavMailCookieSpec");
         return httpClient;
@@ -843,4 +852,8 @@ public final class DavGatewayHttpClientFacade {
             }
         }
     }
+
+    public static String getUserAgent() {
+        return Settings.getProperty("davmail.userAgent", IE_USER_AGENT);
+    }
 }
