@@ -103,12 +103,15 @@ public class SettingsFrame extends JFrame {
     protected JCheckBox imapAutoExpungeCheckBox;
     protected JCheckBox enableKeepAliveCheckBox;
     protected JCheckBox popMarkReadOnRetrCheckBox;
-    protected JComboBox enableEwsComboBox;
+    protected JComboBox modeComboBox;
     protected JCheckBox enableKerberosCheckBox;
     protected JTextField folderSizeLimitField;
     protected JCheckBox smtpSaveInSentCheckBox;
 	protected JCheckBox imapAlwaysApproxMsgSizeCheckBox;
 
+	protected JTextField oauthClientIdField;
+    protected JTextField oauthRedirectUriField;
+
     JCheckBox osxHideFromDockCheckBox;
 
     protected void addSettingComponent(JPanel panel, String label, JComponent component) {
@@ -122,6 +125,7 @@ public class SettingsFrame extends JFrame {
         panel.add(fieldLabel);
         component.setMaximumSize(component.getPreferredSize());
         JPanel innerPanel = new JPanel();
+        innerPanel.setBorder(BorderFactory.createEmptyBorder(1,0,0,0));
         innerPanel.setLayout(new BoxLayout(innerPanel, BoxLayout.X_AXIS));
         innerPanel.add(component);
         panel.add(innerPanel);
@@ -153,8 +157,28 @@ public class SettingsFrame extends JFrame {
         JPanel settingsPanel = new JPanel(new GridLayout(7, 2));
         settingsPanel.setBorder(BorderFactory.createTitledBorder(BundleMessage.format("UI_GATEWAY")));
 
-        enableEwsComboBox = new JComboBox(new String[]{WEBDAV, EWS, AUTO});
-        setEwsModeSelectedItem(Settings.getProperty("davmail.enableEws", "auto"));
+        modeComboBox = new JComboBox();
+        modeComboBox.addItem(Settings.EWS);
+        modeComboBox.addItem(Settings.O365);
+        modeComboBox.addItem(Settings.O365_MODERN);
+        if (isJavaFXAvailable()) {
+            modeComboBox.addItem(Settings.O365_INTERACTIVE);
+        }
+        modeComboBox.addItem(Settings.WEBDAV);
+        modeComboBox.addItem(Settings.AUTO);
+        modeComboBox.setSelectedItem(Settings.getProperty("davmail.mode", Settings.EWS));
+        modeComboBox.addActionListener(new ActionListener() {
+            public void actionPerformed(ActionEvent evt) {
+                String selectedItem = (String)modeComboBox.getSelectedItem();
+                modeComboBox.setToolTipText(BundleMessage.format("UI_"+selectedItem+"_HELP"));
+                if (selectedItem != null && selectedItem.startsWith("O365")) {
+                    urlField.setEnabled(false);
+                    urlField.setText(Settings.O365_URL);
+                } else {
+                    urlField.setEnabled(true);
+                }
+            }
+        });
         urlField = new JTextField(Settings.getProperty("davmail.url"), 20);
         popPortField = new JTextField(Settings.getProperty("davmail.popPort"), 4);
         popPortCheckBox = new JCheckBox();
@@ -221,7 +245,7 @@ public class SettingsFrame extends JFrame {
             }
         });
 
-        addSettingComponent(settingsPanel, BundleMessage.format("UI_ENABLE_EWS"), enableEwsComboBox,
+        addSettingComponent(settingsPanel, BundleMessage.format("UI_ENABLE_EWS"), modeComboBox,
                 BundleMessage.format("UI_ENABLE_EWS_HELP"));
         addSettingComponent(settingsPanel, BundleMessage.format("UI_OWA_URL"), urlField, BundleMessage.format("UI_OWA_URL_HELP"));
         addPortSettingComponent(settingsPanel, BundleMessage.format("UI_POP_PORT"), popPortField, popPortCheckBox,
@@ -237,6 +261,17 @@ public class SettingsFrame extends JFrame {
         return settingsPanel;
     }
 
+    private boolean isJavaFXAvailable() {
+        boolean isJavaFXAvailable = false;
+        try {
+            Class.forName("javafx.embed.swing.JFXPanel");
+            isJavaFXAvailable = true;
+        } catch (ClassNotFoundException e) {
+            // ignore
+        }
+        return isJavaFXAvailable;
+    }
+
     protected JPanel getDelaysPanel() {
         JPanel delaysPanel = new JPanel(new GridLayout(4, 2));
         delaysPanel.setBorder(BorderFactory.createTitledBorder(BundleMessage.format("UI_DELAYS")));
@@ -326,9 +361,9 @@ public class SettingsFrame extends JFrame {
 
         keystoreTypeCombo = new JComboBox(new String[]{"JKS", "PKCS12"});
         keystoreTypeCombo.setSelectedItem(Settings.getProperty("davmail.ssl.keystoreType"));
-        keystoreFileField = new JTextField(Settings.getProperty("davmail.ssl.keystoreFile"), 17);
-        keystorePassField = new JPasswordField(Settings.getProperty("davmail.ssl.keystorePass"), 15);
-        keyPassField = new JPasswordField(Settings.getProperty("davmail.ssl.keyPass"), 15);
+        keystoreFileField = new JTextField(Settings.getProperty("davmail.ssl.keystoreFile"), 20);
+        keystorePassField = new JPasswordField(Settings.getProperty("davmail.ssl.keystorePass"), 20);
+        keyPassField = new JPasswordField(Settings.getProperty("davmail.ssl.keyPass"), 20);
 
         addSettingComponent(keyStorePanel, BundleMessage.format("UI_KEY_STORE_TYPE"), keystoreTypeCombo,
                 BundleMessage.format("UI_KEY_STORE_TYPE_HELP"));
@@ -353,11 +388,11 @@ public class SettingsFrame extends JFrame {
             clientKeystoreTypeCombo = new JComboBox(new String[]{"PKCS11", "JKS", "PKCS12"});
         }
         clientKeystoreTypeCombo.setSelectedItem(Settings.getProperty("davmail.ssl.clientKeystoreType"));
-        clientKeystoreFileField = new JTextField(Settings.getProperty("davmail.ssl.clientKeystoreFile"), 17);
-        clientKeystorePassField = new JPasswordField(Settings.getProperty("davmail.ssl.clientKeystorePass"), 15);
+        clientKeystoreFileField = new JTextField(Settings.getProperty("davmail.ssl.clientKeystoreFile"), 20);
+        clientKeystorePassField = new JPasswordField(Settings.getProperty("davmail.ssl.clientKeystorePass"), 20);
 
-        pkcs11LibraryField = new JTextField(Settings.getProperty("davmail.ssl.pkcs11Library"), 17);
-        pkcs11ConfigField = new JTextArea(2, 17);
+        pkcs11LibraryField = new JTextField(Settings.getProperty("davmail.ssl.pkcs11Library"), 20);
+        pkcs11ConfigField = new JTextArea(2, 20);
         pkcs11ConfigField.setText(Settings.getProperty("davmail.ssl.pkcs11Config"));
         pkcs11ConfigField.setBorder(pkcs11LibraryField.getBorder());
         pkcs11ConfigField.setFont(pkcs11LibraryField.getFont());
@@ -375,7 +410,9 @@ public class SettingsFrame extends JFrame {
                 BundleMessage.format("UI_CLIENT_KEY_STORE_HELP"));
         addSettingComponent(clientKeystoreFilePanel, BundleMessage.format("UI_CLIENT_KEY_STORE_PASSWORD"), clientKeystorePassField,
                 BundleMessage.format("UI_CLIENT_KEY_STORE_PASSWORD_HELP"));
-        cardPanel.add(clientKeystoreFilePanel, "FILE");
+        JPanel wrapperPanel = new JPanel();
+        wrapperPanel.add(clientKeystoreFilePanel);
+        cardPanel.add(wrapperPanel, "FILE");
 
         JPanel pkcs11Panel = new JPanel(new GridLayout(2, 2));
         addSettingComponent(pkcs11Panel, BundleMessage.format("UI_PKCS11_LIBRARY"), pkcs11LibraryField,
@@ -400,6 +437,21 @@ public class SettingsFrame extends JFrame {
         return clientKeystorePanel;
     }
 
+    protected JPanel getOauthPanel() {
+        JPanel oAuthPanel = new JPanel(new GridLayout(2, 2));
+        oAuthPanel.setBorder(BorderFactory.createTitledBorder(BundleMessage.format("UI_OAUTH")));
+
+        oauthClientIdField = new JTextField(Settings.getProperty("davmail.oauth.clientId"), 20);
+        oauthRedirectUriField = new JTextField(Settings.getProperty("davmail.oauth.redirectUri"), 20);
+
+        addSettingComponent(oAuthPanel, BundleMessage.format("UI_OAUTH_CLIENTID"), oauthClientIdField,
+                BundleMessage.format("UI_OAUTH_CLIENTID_HELP"));
+        addSettingComponent(oAuthPanel, BundleMessage.format("UI_OAUTH_REDIRECTURI"), oauthRedirectUriField,
+                BundleMessage.format("UI_OAUTH_REDIRECTURI_HELP"));
+        updateMaximumSize(oAuthPanel);
+        return oAuthPanel;
+    }
+
     protected JPanel getNetworkSettingsPanel() {
         JPanel networkSettingsPanel = new JPanel(new GridLayout(4, 2));
         networkSettingsPanel.setBorder(BorderFactory.createTitledBorder(BundleMessage.format("UI_NETWORK")));
@@ -424,20 +476,6 @@ public class SettingsFrame extends JFrame {
         return networkSettingsPanel;
     }
 
-    protected static final String WEBDAV = "WebDav";
-    protected static final String EWS = "EWS";
-    protected static final String AUTO = "Auto";
-
-    protected void setEwsModeSelectedItem(String ewsMode) {
-        if ("true".equals(ewsMode)) {
-            enableEwsComboBox.setSelectedItem(EWS);
-        } else if ("false".equals(ewsMode)) {
-            enableEwsComboBox.setSelectedItem(WEBDAV);
-        } else {
-            enableEwsComboBox.setSelectedItem(AUTO);
-        }
-    }
-
     protected JPanel getOtherSettingsPanel() {
         JPanel otherSettingsPanel = new JPanel(new GridLayout(15, 2));
         otherSettingsPanel.setBorder(BorderFactory.createTitledBorder(BundleMessage.format("UI_OTHER")));
@@ -633,7 +671,7 @@ public class SettingsFrame extends JFrame {
         imapAlwaysApproxMsgSizeCheckBox.setSelected(Settings.getBooleanProperty("davmail.imapAlwaysApproxMsgSize", false));
         enableKeepAliveCheckBox.setSelected(Settings.getBooleanProperty("davmail.enableKeepAlive", false));
         popMarkReadOnRetrCheckBox.setSelected(Settings.getBooleanProperty("davmail.popMarkReadOnRetr", false));
-        setEwsModeSelectedItem(Settings.getProperty("davmail.enableEws", "auto"));
+        modeComboBox.setSelectedItem(Settings.getProperty("davmail.mode", Settings.EWS));
         smtpSaveInSentCheckBox.setSelected(Settings.getBooleanProperty("davmail.smtpSaveInSent", true));
         enableKerberosCheckBox.setSelected(Settings.getBooleanProperty("davmail.enableKerberos", false));
         folderSizeLimitField.setText(Settings.getProperty("davmail.folderSizeLimit"));
@@ -647,6 +685,9 @@ public class SettingsFrame extends JFrame {
         pkcs11LibraryField.setText(Settings.getProperty("davmail.ssl.pkcs11Library"));
         pkcs11ConfigField.setText(Settings.getProperty("davmail.ssl.pkcs11Config"));
 
+        oauthClientIdField.setText(Settings.getProperty("davmail.oauth.clientId"));
+        oauthRedirectUriField.setText(Settings.getProperty("davmail.oauth.redirectUri"));
+
         rootLoggingLevelField.setSelectedItem(Settings.getLoggingLevel("rootLogger"));
         davmailLoggingLevelField.setSelectedItem(Settings.getLoggingLevel("davmail"));
         httpclientLoggingLevelField.setSelectedItem(Settings.getLoggingLevel("org.apache.commons.httpclient"));
@@ -718,6 +759,7 @@ public class SettingsFrame extends JFrame {
         encryptionPanel.setLayout(new BoxLayout(encryptionPanel, BoxLayout.Y_AXIS));
         encryptionPanel.add(getKeystorePanel());
         encryptionPanel.add(getSmartCardPanel());
+        encryptionPanel.add(getOauthPanel());
         // empty panel
         encryptionPanel.add(new JPanel());
         tabbedPane.add(BundleMessage.format("UI_TAB_ENCRYPTION"), encryptionPanel);
@@ -798,16 +840,8 @@ public class SettingsFrame extends JFrame {
                 Settings.setProperty("davmail.imapAlwaysApproxMsgSize", String.valueOf(imapAlwaysApproxMsgSizeCheckBox.isSelected()));
                 Settings.setProperty("davmail.enableKeepAlive", String.valueOf(enableKeepAliveCheckBox.isSelected()));
                 Settings.setProperty("davmail.popMarkReadOnRetr", String.valueOf(popMarkReadOnRetrCheckBox.isSelected()));
-                String selectedEwsMode = (String) enableEwsComboBox.getSelectedItem();
-                String enableEws;
-                if (EWS.equals(selectedEwsMode)) {
-                    enableEws = "true";
-                } else if (WEBDAV.equals(selectedEwsMode)) {
-                    enableEws = "false";
-                } else {
-                    enableEws = "auto";
-                }
-                Settings.setProperty("davmail.enableEws", enableEws);
+
+                Settings.setProperty("davmail.mode", (String) modeComboBox.getSelectedItem());
                 Settings.setProperty("davmail.enableKerberos", String.valueOf(enableKerberosCheckBox.isSelected()));
                 Settings.setProperty("davmail.folderSizeLimit", folderSizeLimitField.getText());
                 Settings.setProperty("davmail.smtpSaveInSent", String.valueOf(smtpSaveInSentCheckBox.isSelected()));
@@ -823,6 +857,9 @@ public class SettingsFrame extends JFrame {
                 Settings.setProperty("davmail.ssl.pkcs11Library", pkcs11LibraryField.getText());
                 Settings.setProperty("davmail.ssl.pkcs11Config", pkcs11ConfigField.getText());
 
+                Settings.setProperty("davmail.oauth.clientId", oauthClientIdField.getText());
+                Settings.setProperty("davmail.oauth.redirectUri", oauthRedirectUriField.getText());
+
                 Settings.setLoggingLevel("rootLogger", (Level) rootLoggingLevelField.getSelectedItem());
                 Settings.setLoggingLevel("davmail", (Level) davmailLoggingLevelField.getSelectedItem());
                 Settings.setLoggingLevel("org.apache.commons.httpclient", (Level) httpclientLoggingLevelField.getSelectedItem());
